// SPDX-FileCopyrightText: 2023 Joshua Goins <josh@redstrate.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "artconfigwindow.h"

#include <KLocalizedString>
#include <QFile>
#include <QFormLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QJsonArray>
#include <QJsonDocument>
#include <QLineEdit>
#include <QPushButton>

ArtConfigWindow::ArtConfigWindow(const QString &filename, const QString &definitionDirectory, QWidget *parent)
    : QDialog(parent)
{
    setWindowModality(Qt::WindowModality::WindowModal);
    setWindowTitle(filename);

    auto mainLayout = new QHBoxLayout();
    setLayout(mainLayout);

    auto formLayout = new QFormLayout();
    auto formLayoutWidget = new QWidget();
    formLayoutWidget->setLayout(formLayout);
    mainLayout->addWidget(formLayoutWidget);

    model = new FeaturedArtModel(definitionDirectory);

    auto galleryListView = new QListView();
    galleryListView->setModel(model);
    formLayout->addWidget(galleryListView);

    m_newBannerEdit = new QLineEdit();
    formLayout->addRow(i18nc("@label:textbox", "New Banner"), m_newBannerEdit);

    m_commissionsOpen = new QCheckBox();
    formLayout->addRow(i18nc("@option:check", "Commissions Open"), m_commissionsOpen);

    auto bottomButtonLayout = new QHBoxLayout();
    formLayout->addRow(bottomButtonLayout);

    auto cancelButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-close")), i18nc("@action:button", "Cancel"));
    connect(cancelButton, &QPushButton::clicked, this, &ArtConfigWindow::close);
    bottomButtonLayout->addWidget(cancelButton);
    bottomButtonLayout->addStretch(1);

    saveButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-ok")), i18nc("@action:button", "Save"));
    connect(saveButton, &QPushButton::clicked, this, [this, filename] {
        saveData(filename);
    });
    saveButton->setEnabled(false);
    bottomButtonLayout->addWidget(saveButton);

    if (QFile::exists(filename)) {
        loadData(filename);
    }
}

void ArtConfigWindow::loadData(const QString &filename)
{
    qDebug() << "Loading data from" << filename;

    QFile artFile(filename);
    artFile.open(QFile::ReadOnly);

    const QJsonDocument artJson = QJsonDocument::fromJson(artFile.readAll());

    m_newBannerEdit->setText(artJson[QStringLiteral("new-banner")].toString());
    m_commissionsOpen->setChecked(artJson[QStringLiteral("commissions")].toBool());

    connect(model, &ArtModel::loadingFinished, this, [this, artJson] {
        saveButton->setEnabled(true);

        QStringList featuredList;
        for (auto featuredId : artJson["featured"].toArray()) {
            featuredList.push_back(featuredId.toString());
        }
        model->setFeaturedItems(featuredList);
    });
}

void ArtConfigWindow::saveData(const QString &filename)
{
    qDebug() << "Saving data to" << filename;

    QJsonObject object;
    object[QStringLiteral("new-banner")] = m_newBannerEdit->text();
    object[QStringLiteral("commissions")] = m_commissionsOpen->isChecked();

    QJsonArray featuredArray;
    for (int i = 0; i < model->rowCount({}); i++) {
        const auto checkState = model->data(model->index(i, 0), Qt::CheckStateRole).value<Qt::CheckState>();
        if (checkState == Qt::Checked) {
            QFileInfo fileInfo(model->data(model->index(i, 1), Qt::DisplayRole).toString());
            featuredArray.push_back(fileInfo.baseName());
        }
    }

    object["featured"] = featuredArray;

    const QJsonDocument jsonDoc(object);

    QFile file(filename);
    file.open(QFile::WriteOnly);
    file.write(jsonDoc.toJson());
    file.close();

    close();
}
