// SPDX-FileCopyrightText: 2024 Joshua Goins <josh@redstrate.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "characterdetailwindow.h"
#include "imagelabel.h"

#include <KLocalizedString>
#include <QFile>
#include <QFileInfo>
#include <QFormLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QJsonArray>
#include <QJsonDocument>
#include <QLineEdit>
#include <QPushButton>
#include <QTextEdit>

CharacterDetailWindow::CharacterDetailWindow(const QString &filename, QWidget *parent)
    : QDialog(parent)
{
    setMinimumWidth(800);
    setMinimumHeight(600);
    setWindowModality(Qt::WindowModality::WindowModal);
    setWindowTitle(filename);

    QFileInfo info(filename);
    const QString withoutExtension = info.completeBaseName();

    auto mainLayout = new QHBoxLayout();
    setLayout(mainLayout);

    auto formLayout = new QFormLayout();
    auto formLayoutWidget = new QWidget();
    formLayoutWidget->setMaximumWidth(450);
    formLayoutWidget->setLayout(formLayout);
    mainLayout->addWidget(formLayoutWidget);

    auto previewBox = new QGroupBox(i18nc("@title:group", "Preview"));
    mainLayout->addWidget(previewBox);

    auto previewLayout = new QVBoxLayout();
    previewBox->setLayout(previewLayout);

    auto imageView = new ImageLabel();
    previewLayout->addWidget(imageView);

    m_nameEdit = new QLineEdit();
    formLayout->addRow(i18nc("@label:textbox", "Title"), m_nameEdit);

    m_pronounsEdit = new QLineEdit();
    formLayout->addRow(i18nc("@label:textbox", "Pronouns"), m_pronounsEdit);

    m_descriptionEdit = new QTextEdit();
    m_descriptionEdit->setAcceptRichText(false);
    formLayout->addRow(i18nc("@label:textbox", "Description"), m_descriptionEdit);

    m_ageEdit = new QLineEdit();
    formLayout->addRow(i18nc("@label:textbox", "Age"), m_ageEdit);

    m_originEdit = new QLineEdit();
    formLayout->addRow(i18nc("@label:textbox", "Origin"), m_originEdit);

    m_originalCharacterEdit = new QCheckBox();
    formLayout->addRow(i18nc("@label:checkbox", "Original Character"), m_originalCharacterEdit);

    auto bottomButtonLayout = new QHBoxLayout();
    formLayout->addRow(bottomButtonLayout);

    auto cancelButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-close")), i18nc("@action:button", "Cancel"));
    connect(cancelButton, &QPushButton::clicked, this, &CharacterDetailWindow::close);
    bottomButtonLayout->addWidget(cancelButton);
    bottomButtonLayout->addStretch(1);

    auto saveButton = new QPushButton(QIcon::fromTheme(QStringLiteral("dialog-ok")), i18nc("@action:button", "Save"));
    connect(saveButton, &QPushButton::clicked, this, [this, filename] {
        saveData(filename);
    });
    bottomButtonLayout->addWidget(saveButton);

    if (QFile::exists(filename)) {
        loadData(filename);
    }
}

void CharacterDetailWindow::loadData(const QString &filename)
{
    qDebug() << "Loading data from" << filename;

    QFile artFile(filename);
    artFile.open(QFile::ReadOnly);

    const QJsonDocument artJson = QJsonDocument::fromJson(artFile.readAll());

    if (artJson.object().contains(QStringLiteral("name"))) {
        m_nameEdit->setText(artJson[QStringLiteral("name")].toString());
    }

    if (artJson.object().contains(QStringLiteral("pronouns"))) {
        m_pronounsEdit->setText(artJson[QStringLiteral("pronouns")].toString());
    }

    if (artJson.object().contains(QStringLiteral("age"))) {
        m_ageEdit->setText(artJson[QStringLiteral("age")].toString());
    }

    if (artJson.object().contains(QStringLiteral("description"))) {
        m_descriptionEdit->setText(artJson[QStringLiteral("description")].toString());
    }

    if (artJson.object().contains(QStringLiteral("origin"))) {
        m_originEdit->setText(artJson[QStringLiteral("origin")].toString());
    }

    if (artJson.object().contains(QStringLiteral("original"))) {
        m_originalCharacterEdit->setChecked(artJson[QStringLiteral("original")].toBool());
    }
}

void CharacterDetailWindow::saveData(const QString &filename)
{
    qDebug() << "Saving data to" << filename;

    QJsonObject object;

    if (!m_nameEdit->text().isEmpty()) {
        object[QStringLiteral("name")] = m_nameEdit->text();
    }

    if (!m_pronounsEdit->text().isEmpty()) {
        object[QStringLiteral("pronouns")] = m_pronounsEdit->text();
    }

    if (!m_ageEdit->text().isEmpty()) {
        object[QStringLiteral("age")] = m_ageEdit->text();
    }

    if (!m_descriptionEdit->document()->toPlainText().isEmpty()) {
        object[QStringLiteral("description")] = m_descriptionEdit->document()->toPlainText();
    }

    if (!m_originEdit->text().isEmpty()) {
        object[QStringLiteral("origin")] = m_originEdit->text();
    }

    object[QStringLiteral("original")] = m_originalCharacterEdit->isChecked();

    const QJsonDocument jsonDoc(object);

    QFile file(filename);
    file.open(QFile::WriteOnly);
    file.write(jsonDoc.toJson());
    file.close();

    close();
}
