// SPDX-FileCopyrightText: 2024 Joshua Goins <josh@redstrate.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "charactermodel.h"

#include <KLocalizedString>
#include <QCoroTask>
#include <QDirIterator>
#include <QFile>
#include <QFileInfo>
#include <QJsonDocument>
#include <QPixmap>
#include <QtConcurrent>

CharacterModel::CharacterModel(const QDir &definitionDirectory, QObject *parent)
    : QAbstractTableModel(parent)
{
    charactersFuture = new QFutureWatcher<Character *>(this);
    connect(charactersFuture, &QFutureWatcher<Character *>::resultReadyAt, this, &CharacterModel::characterFinished);
    connect(charactersFuture, &QFutureWatcher<Character *>::finished, this, &CharacterModel::finished);

    QVector<QString> characterList;

    QDirIterator it(definitionDirectory);
    while (it.hasNext()) {
        QFileInfo info(it.next());
        if (!info.isFile()) {
            continue;
        }

        characterList.push_back(definitionDirectory.absoluteFilePath(info.baseName()));

        beginInsertRows(QModelIndex(), static_cast<int>(m_characters.size()), static_cast<int>(m_characters.size() + 1));

        m_characters.push_back({});

        endInsertRows();
    }

    const std::function<QCoro::Task<Character *>(const QString &info)> loadPiece = [](const QString &info) -> QCoro::Task<Character *> {
        auto p = new Character(info);

        co_return p;
    };

    charactersFuture->setFuture(QtConcurrent::mapped(characterList, [loadPiece](const QString &info) -> Character * {
        return QCoro::waitFor(loadPiece(info));
    }));
}

int CharacterModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return static_cast<int>(m_characters.size());
}

int CharacterModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return 1;
}

QVariant CharacterModel::data(const QModelIndex &index, const int role) const
{
    if (!index.isValid()) {
        return {};
    }

    if (m_characters[index.row()] == nullptr) {
        return {};
    }

    if (role == Qt::DisplayRole) {
        switch (index.column()) {
        case 0: {
            return m_characters[index.row()]->name;
        }
        }
    } else if (role == Qt::UserRole) {
        return QVariant::fromValue(m_characters[index.row()]);
    }

    return {};
}

QVariant CharacterModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Orientation::Horizontal && role == Qt::DisplayRole) {
        switch (section) {
        case 0:
            return i18nc("@title:column", "Name");
        default:
            Q_UNREACHABLE();
        }
    }

    return QAbstractTableModel::headerData(section, orientation, role);
}

void CharacterModel::characterFinished(const int row)
{
    m_characters[row] = charactersFuture->resultAt(row);

    Q_EMIT dataChanged(index(row, 0), index(row + 1, 0));
}

void CharacterModel::finished()
{
    std::sort(m_characters.begin(), m_characters.end(), [](const Character *a, const Character *b) {
        return a->name > b->name;
    });

    Q_EMIT dataChanged(index(0, 0), index(static_cast<int>(m_characters.size()), 3));

    Q_EMIT loadingFinished();
}

Character::Character(const QString &filename)
{
    this->filename = filename + ".json";

    QFile artFile(this->filename);
    artFile.open(QFile::ReadOnly);

    const QJsonDocument artJson = QJsonDocument::fromJson(artFile.readAll());

    if (artJson.object().contains(QStringLiteral("name"))) {
        name = artJson.object()[QStringLiteral("name")].toString();
    }
}
